### 实验十四 板间WIFI通信

看过前面的WIFI连接和温湿度传感器的用法之后，这里我们将他们结合一下，做一个板间通信

板间WIFI通信的原理是其中一块板子连接wifi然后向IP地址里发送数据，另一块板子连接wifi从IP地址里获取数据。

#### 代码讲解

作为采集数据并发送的模块烧录以下程序

![image-20221206142213272](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十四 板间WIFI通信\通信1.png)

需要注意的函数有

**server.on("/temperature", HTTP_GET, [](AsyncWebServerRequest *request）**

**request->send_P(200, "text/plain", readTemp().c_str())**

这个是将对应数据传送到WIFI位置的函数，在操作时建议仅修改传输数据的部分

**server.begin()**

服务器初始化函数

接收数据的板子烧录以下程序

![image-20221206142537409](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十四 板间WIFI通信\通信2.png)

这个程序的作用是从云端获取刚刚另外一块板子传输上去的数据，数据存储在IP地址位置，并且在串口和OLED上面打印所收到的数据

这里面用到的函数有

**WiFi.localIP()**

WiFi的IP地址

**httpGETRequest(serverNameTemp);**

从服务器IP地址获取数据，括号内为IP地址

**http.begin(client, serverName)**

服务器初始化

**http.GET()**

回调参数获取

**http.getString()**

服务器获取字符串

**http.end();**

结束访问

#### 程序源码

**通信板1**

```
#include "WiFi.h"
#include "ESPAsyncWebServer.h"
#include <DHT.h>
#include <DHT_U.h>
#define DHTPIN 16   
#define DHTTYPE DHT11  
DHT dht(DHTPIN, DHTTYPE); 

// Set your access point network credentials
const char* ssid = "zhou";
const char* password = "12345678.";


// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

String readTemp() {
  return String(dht.readHumidity());
  //return String(1.8 * bme.readTemperature() + 32);
}

String readHumi() {
  return String(dht.readTemperature());
}

void setup(){
  // Serial port for debugging purposes
  Serial.begin(115200);
  Serial.println();
  dht.begin(); 
  // Setting the ESP as an access point
  Serial.print("Setting AP (Access Point)…");
  // Remove the password parameter, if you want the AP (Access Point) to be open
  WiFi.softAP(ssid, password);

  IPAddress IP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(IP);

  server.on("/temperature", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", readTemp().c_str());
  });
  server.on("/humidity", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", readHumi().c_str());
  });


  server.begin();
}

void loop(){

}
```

**通信板2**

```
#include <WiFi.h>
#include <HTTPClient.h>

const char* ssid = "zhou";
const char* password = "12345678.";

//Your IP address or domain name with URL path
const char* serverNameTemp = "http://192.168.4.1/temperature";
const char* serverNameHumi = "http://192.168.4.1/humidity";


#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define SDA_PIN 17                       
#define SCL_PIN 16

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

String temperature;
String humidity;

unsigned long previousMillis = 0;
const long interval = 5000; 

void setup() {
  Serial.begin(115200);
  Wire.begin(SDA_PIN,SCL_PIN); 
  // Address 0x3C for 128x64, you might need to change this value (use an I2C scanner)
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  display.clearDisplay();
  display.setTextColor(WHITE);

  WiFi.begin(ssid, password);
  Serial.println("Connecting");
  while(WiFi.status() != WL_CONNECTED) { 
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());
}

void loop() {
  unsigned long currentMillis = millis();

  if(currentMillis - previousMillis >= interval) {
     // Check WiFi connection status
    if(WiFi.status()== WL_CONNECTED ){ 
      temperature = httpGETRequest(serverNameTemp);
      humidity = httpGETRequest(serverNameHumi);
      Serial.println("Temperature: " + temperature + " *C - Humidity: " + humidity + " % ");
      
      display.clearDisplay();
      
      // display temperature
      display.setTextSize(2);
      display.setTextColor(WHITE);
      display.setCursor(0,0);
      display.print("T: ");
      display.print(temperature);
      display.print(" ");
      display.setTextSize(1);
      display.cp437(true);
      display.write(248);
      display.setTextSize(2);
      display.print("C");
      
      // display humidity
      display.setTextSize(2);
      display.setCursor(0, 25);
      display.print("H: ");
      display.print(humidity);
      display.print(" %"); 
      
      display.display();
      
      // save the last HTTP GET Request
      previousMillis = currentMillis;
    }
    else {
      Serial.println("WiFi Disconnected");
    }
  }
}

String httpGETRequest(const char* serverName) {
  WiFiClient client;
  HTTPClient http;
    
  // Your Domain name with URL path or IP address with path
  http.begin(client, serverName);

  // Send HTTP POST request
  int httpResponseCode = http.GET();

  String payload = "--"; 

  if (httpResponseCode>0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();

  return payload;
}
```

